<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayerGa4;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
use Flat101\EnhancedEcommerce\Helper\Config;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Model\Session;
use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Sales\Model\Order\Item;
use Flat101\EnhancedEcommerce\Plugin\ShippingInformation;

class PurchaseView implements DataLayerInterface
{
    use UnsetProperty;

    public const CODE = 'purchase-view-ga4';

    private const EVENT = 'purchase';

    /** @var Session */
    private $session;

    /** @var CheckoutSession */
    private $checkoutSession;

    /** @var Generator\Product */
    private $productGenerator;

    /** @var Config */
    private $configHelper;

    public function __construct(
        Session $session,
        CheckoutSession $checkoutSession,
        Generator\Product $productGenerator,
        Config $configHelper
    ) {
        $this->session = $session;
        $this->checkoutSession = $checkoutSession;
        $this->productGenerator = $productGenerator;
        $this->configHelper = $configHelper;
    }

    public function getCode(): string
    {
        return static::CODE;
    }

    public function getLayer()
    {
        $order = $this->checkoutSession->getLastRealOrder();

        $excludedPaymentMethods = $this->configHelper->getExcludedPaymentMethods($order->getStoreId());
        if (!in_array($order->getPayment()->getMethodInstance()->getCode(), $excludedPaymentMethods)) {
            return null;
        }

        $productItems = [];

        /** @var Item $item */
        foreach ($order->getAllVisibleItems() as $item) {
            $productItems[] = $this->productGenerator->generate($item->getProduct(), $item->getQtyOrdered(), $item);
        }

        $purchaseDto = new Dto\Purchase\Purchase();
        $purchaseDto->transaction_id = $order->getIncrementId();
        $purchaseDto->value = floatval($order->getGrandTotal());
        $purchaseDto->tax = floatval($order->getTaxAmount());
        $purchaseDto->shipping = floatval($order->getShippingAmount());
        $purchaseDto->currency = $order->getOrderCurrencyCode();
        $purchaseDto->coupon = $order->getCouponCode() ?: '';
        $purchaseDto->items = $productItems;

        $ecommerce = new Dto\Ecommerce();
        $ecommerce->params = $purchaseDto;
        $ecommerce->name = self::EVENT;

        return $ecommerce;
    }
}
