<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Plugin;

use Flat101\EnhancedEcommerce\Helper\MeasurementProtocol;
use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\CheckoutStep;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Generator\CheckoutStep as CheckoutStepGa4;
use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\CheckoutOptionStep;
use Flat101\EnhancedEcommerce\Model\DataLayer\CheckoutView;
use Flat101\EnhancedEcommerce\Model\Session AS CustomSession;
use Magento\Checkout\Api\Data\ShippingInformationInterface;
use Magento\Checkout\Model\ShippingInformationManagement;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Customer\Model\Session AS CustomerModelSession;

class ShippingInformation
{
    const PAYMENT_STEP = 2;
    const PAYMENT_STEP_LABEL = 'step 2 - Payment info';
    const EVENT_NAME_BEGIN_GA4 = 'begin_checkout';

    /** @var CartRepositoryInterface */
    private $quoteRepository;

    /** @var CustomSession */
    private $customSession;

    /** @var CheckoutStep */
    private $checkoutStep;

    /** @var CheckoutStepGa4 */
    private $checkoutStepGa4;

    /** @var CheckoutOptionStep */
    private $checkoutOptionStep;

    /** @var CustomerModelSession */
    private $customerSession;

    /** @var MeasurementProtocol */
    private $measurementProtocol;

    /** @var CookieManagerInterface */
    protected $cookieManager;

    /** @var StoreManagerInterface */
    protected $storeManager;

    public function __construct(
        CartRepositoryInterface $quoteRepository,
        MeasurementProtocol $measurementProtocol,
        CustomSession $customSession,
        CookieManagerInterface $cookieManager,
        CheckoutStep $checkoutStep,
        CheckoutStepGa4 $checkoutStepGa4,
        CheckoutOptionStep $checkoutOptionStep,
        CustomerModelSession $session,
        StoreManagerInterface $storeManager
    ) {
        $this->quoteRepository = $quoteRepository;
        $this->customSession = $customSession;
        $this->checkoutStep = $checkoutStep;
        $this->checkoutStepGa4 = $checkoutStepGa4;
        $this->checkoutOptionStep = $checkoutOptionStep;
        $this->customerSession = $session;
        $this->measurementProtocol = $measurementProtocol;
        $this->cookieManager = $cookieManager;
        $this->storeManager = $storeManager;
    }

    public function afterSaveAddressInformation(
        ShippingInformationManagement $object,
        $result,
        $cartId,
        ShippingInformationInterface $addressInformation
    ) {
        $this->customSession->setCheckoutStep(
            $this->checkoutStepGa4->onCheckoutStep(self::EVENT_NAME_BEGIN_GA4)
        );

        $cid = $this->cookieManager->getCookie('_ga');
        $shipping = $this->checkoutStepGa4->onCheckoutStep(self::EVENT_NAME_BEGIN_GA4);
        $this->measurementProtocol->sendDataToGa4($shipping, $this->getStoreId(), $cid);

        $this->customSession->setCheckoutStep(
            $this->checkoutStep->onCheckoutStep(self::PAYMENT_STEP, null, self::PAYMENT_STEP_LABEL)
        );

        return $result;
    }

    public function getStoreId()
    {
        return $this->storeManager->getStore()->getId();
    }
}
